@@ -527,6 +527,10 @@ def all_off_keypad():
         else:
             if zonepwralloff[int(count)] == 1 and ALL_OFF_RECALL:
                 zonepwr[int(count)] = 1
+        if count in media_zonetable:
+            callbackobj = media_zonetable.index(count)
+            callbackint = int(callbackobj)
+            media_callback[callbackint]()
 
 def keypad_lock_recv(match):
     zone = abs(int(match[1]))
@@ -562,14 +566,6 @@ def get_nuvo(port_url, baud, all_off_recall):
         def __init__(self, port_url):
             global ALL_OFF_RECALL
             ALL_OFF_RECALL = all_off_recall
-            self._port = serial.serial_for_url(port_url, do_not_open=True)
-            self._port.baudrate = int(baud)
-            self._port.stopbits = serial.STOPBITS_ONE
-            self._port.bytesize = serial.EIGHTBITS
-            self._port.parity = serial.PARITY_NONE
-            self._port.timeout = TIMEOUT_OP
-            self._port.write_timeout = TIMEOUT_OP
-            self._port.open()
 
         def get_model(self):
             findmodelcount = 0
@@ -594,7 +590,11 @@ def get_nuvo(port_url, baud, all_off_recall):
             #format and send output command
             lineout = "*" + request + "\r"
             _LOGGER.debug('Sending "%s"', request)
-            self._port.write(lineout.encode())
+            try:
+                port.write(lineout.encode())
+            except:
+                _LOGGER.error('Unexpected port error when sending command.')
+                return False
             time.sleep(0.1) #Must wait in between commands for MPU to process
             return True
 
@@ -606,32 +606,50 @@ def get_nuvo(port_url, baud, all_off_recall):
             self._send_request(request)
 
         def _listen():
-            no_data = False
-            receive_buffer = b''
-            message = b''
             start_time = time.time()
             timeout = TIMEOUT_RESPONSE
             _LOGGER.info('Attempting connection - "%s" at %s baud', port_url, baud)
-            try:
-                port = serial.serial_for_url(port_url, do_not_open=True)
-                port.baudrate = int(baud)
-                port.stopbits = serial.STOPBITS_ONE
-                port.bytesize = serial.EIGHTBITS
-                port.parity = serial.PARITY_NONE
-                port.open()
-            except:
-                _LOGGER.error('Could not open serial port.')
+            global listen_init
+            listen_init = 1
             # listen for response
-            while (no_data == False):
-                   # fill buffer until we get term seperator
-               data = port.read(1)
+            def read_data():
+                global listen_init
+                global port
+                try:
+                    port = serial.serial_for_url(port_url, do_not_open=True)
+                    port.baudrate = int(baud)
+                    port.stopbits = serial.STOPBITS_ONE
+                    port.bytesize = serial.EIGHTBITS
+                    port.parity = serial.PARITY_NONE
+                    port.open()
+                except:
+                    if listen_init == 1:
+                         return False
 
-               if data:
-                  receive_buffer += data
-                  if EOL in receive_buffer:
-                      message, sep, receive_buffer = receive_buffer.partition(EOL)
-                      _LOGGER.debug('Received: %s', message)
-                      _parse_response(str(message))
+                no_data = False
+                receive_buffer = b''
+                message = b''
+                listen_init = 0
+                try:
+                    while (no_data == False):
+                    # fill buffer until we get term seperator
+                       data = port.read(1)
+                       if data:
+                          receive_buffer += data
+                          if EOL in receive_buffer:
+                              message, sep, receive_buffer = receive_buffer.partition(EOL)
+                              _LOGGER.debug('Received: %s', message)
+                              _parse_response(str(message))
+                except:
+                    _LOGGER.error('Unexpected port error, retrying.')
+                    time.sleep(5)
+                    try:
+                        port.open()
+                        read_data()
+                    except:
+                        read_data()
+
+            read_data()
 
         SyncThread = threading.Thread(target=_listen, args=(), daemon=True)
         SyncThread.start()
